@@ -26,7 +26,7 @@
 extern "C" {
 #endif
 
-#define LIBGNUTLS_VERSION "0.5.1"
+#define LIBGNUTLS_VERSION "0.5.3"
 
 #include <sys/types.h>
 #include <time.h>
@@ -120,6 +120,7 @@ int gnutls_bye( GNUTLS_STATE state, GNUTLS_CloseRequest how);
 int gnutls_handshake( GNUTLS_STATE state);
 int gnutls_rehandshake( GNUTLS_STATE state);
 
+int gnutls_handshake_get_direction(GNUTLS_STATE state);
 
 GNUTLS_AlertDescription gnutls_alert_get( GNUTLS_STATE state);
 int 		 gnutls_alert_send( GNUTLS_STATE, GNUTLS_AlertLevel, GNUTLS_AlertDescription);
@@ -153,6 +154,7 @@ const char* gnutls_strerror( int error);
  */
 void gnutls_handshake_set_private_extensions(GNUTLS_STATE state, int allow);
 void gnutls_record_set_cbc_protection(GNUTLS_STATE state, int prot);
+void gnutls_handshake_set_exportable_detection(GNUTLS_STATE state, int det);
 void gnutls_handshake_set_rsa_pms_check(GNUTLS_STATE state, int check);
 
 /* Record layer functions.
@@ -176,6 +178,10 @@ int gnutls_kx_set_priority( GNUTLS_STATE state, GNUTLS_LIST);
 int gnutls_protocol_set_priority( GNUTLS_STATE state, GNUTLS_LIST);
 int gnutls_cert_type_set_priority( GNUTLS_STATE state, GNUTLS_LIST);
 
+/* Returns the name of a cipher suite */
+const char *gnutls_cipher_suite_get_name(GNUTLS_KXAlgorithm kx_algorithm,
+	GNUTLS_BulkCipherAlgorithm cipher_algorithm, GNUTLS_MACAlgorithm mac_algorithm);
+
 /* get the currently used protocol version */
 GNUTLS_Version gnutls_protocol_get_version(GNUTLS_STATE state);
 
@@ -256,13 +262,13 @@ int gnutls_certificate_allocate_cred( GNUTLS_CERTIFICATE_CREDENTIALS *sc);
 
 int gnutls_certificate_set_dh_params(GNUTLS_CERTIFICATE_CREDENTIALS res, GNUTLS_DH_PARAMS);
 
-int gnutls_certificate_set_x509_trust_file( GNUTLS_CERTIFICATE_CREDENTIALS res, char* CAFILE, 
+int gnutls_certificate_set_x509_trust_file( GNUTLS_CERTIFICATE_CREDENTIALS res, const char* CAFILE, 
 	GNUTLS_X509_CertificateFmt);
 int gnutls_certificate_set_x509_trust_mem(GNUTLS_CERTIFICATE_CREDENTIALS res, 
 	const gnutls_datum *CA, GNUTLS_X509_CertificateFmt);
 
 int gnutls_certificate_set_x509_key_file( GNUTLS_CERTIFICATE_CREDENTIALS res, 
-	char *CERTFILE, char* KEYFILE, GNUTLS_X509_CertificateFmt);
+	const char *CERTFILE, const char* KEYFILE, GNUTLS_X509_CertificateFmt);
 int gnutls_certificate_set_x509_key_mem(GNUTLS_CERTIFICATE_CREDENTIALS res, 
 	const gnutls_datum* CERT, const gnutls_datum* KEY,
 	GNUTLS_X509_CertificateFmt);
@@ -518,6 +524,7 @@ int gnutls_b64_decode_fmt2( const gnutls_datum *b64_data, const gnutls_datum* re
  */
 #define GNUTLS_E_INIT_LIBEXTRA -82
 #define GNUTLS_E_LIBRARY_VERSION_MISMATCH -82
+#define GNUTLS_E_EXPORT_CIPHER_SUITE -83
 
 #define GNUTLS_E_UNIMPLEMENTED_FEATURE -250
 
