@@ -30,9 +30,9 @@
 # define GNUTLS_CIPHER_FAILED NULL
 #endif
 
-GNUTLS_CIPHER_HANDLE gnutls_cipher_init( BulkCipherAlgorithm cipher, gnutls_datum key, gnutls_datum iv);
-int gnutls_cipher_encrypt(GNUTLS_CIPHER_HANDLE handle, void* text, int textlen);
-int gnutls_cipher_decrypt(GNUTLS_CIPHER_HANDLE handle, void* ciphertext, int ciphertextlen);
-void gnutls_cipher_deinit(GNUTLS_CIPHER_HANDLE handle);
+GNUTLS_CIPHER_HANDLE _gnutls_cipher_init( BulkCipherAlgorithm cipher, gnutls_datum key, gnutls_datum iv);
+int _gnutls_cipher_encrypt(GNUTLS_CIPHER_HANDLE handle, void* text, int textlen);
+int _gnutls_cipher_decrypt(GNUTLS_CIPHER_HANDLE handle, void* ciphertext, int ciphertextlen);
+void _gnutls_cipher_deinit(GNUTLS_CIPHER_HANDLE handle);
 
 #endif /* GNUTLS_CIPHER_INT */
