@@ -19,6 +19,10 @@
  *
  */
 
+/* Functions that are supposed to run after the handshake procedure is
+ * finished. These functions activate the established security parameters.
+ */
+
 #include <gnutls_int.h>
 #include <gnutls_constate.h>
 #include <gnutls_errors.h>
@@ -172,7 +176,8 @@ int _gnutls_set_write_keys(GNUTLS_STATE state)
 	memcpy( dst->session_id, src->session_id, TLS_MAX_SESSION_ID_SIZE); \
 	dst->session_id_size = src->session_id_size; \
 	dst->timestamp = src->timestamp; \
-	dst->max_record_size = src->max_record_size; \
+	dst->max_record_recv_size = src->max_record_recv_size; \
+	dst->max_record_send_size = src->max_record_send_size; \
 	dst->version = src->version; \
 	memcpy( &dst->extensions, &src->extensions, sizeof(TLSExtensions));
 	
