@@ -19,9 +19,10 @@
  *
  */
 
-/* This file contains functions that manipulate a database
+/* This file contains functions that manipulate a database backend
  * for resumed sessions. 
  */
+
 #include "gnutls_int.h"
 #include "gnutls_errors.h"
 #include "gnutls_session.h"
@@ -29,8 +30,6 @@
 #include "debug.h"
 #include <gnutls_session_pack.h>
 
-#define GNUTLS_DBNAME state->gnutls_internals.db_name
-
 /**
   * gnutls_db_set_retrieve_function - Sets the function that will be used to get data
   * @state: is a &GNUTLS_STATE structure.
@@ -210,7 +209,7 @@ gnutls_datum data;
 gnutls_datum key = { session_id, session_id_size };
 int ret;
 
-	if (GNUTLS_DBNAME==NULL && _gnutls_db_func_is_ok(state)!=0) {
+	if (_gnutls_db_func_is_ok(state)!=0) {
 		gnutls_assert();
 		return GNUTLS_E_INVALID_SESSION;
 	}
@@ -254,7 +253,7 @@ int ret = 0;
 		return GNUTLS_E_INVALID_SESSION;
 	}
 	
-	if (GNUTLS_DBNAME==NULL && _gnutls_db_func_is_ok(state)!=0) {
+	if (_gnutls_db_func_is_ok(state)!=0) {
 		return GNUTLS_E_DB_ERROR;
 	}
 	
@@ -300,7 +299,7 @@ int _gnutls_remove_session( GNUTLS_STATE state, gnutls_datum session_id)
 {
 int ret = 0;
 
-	if (GNUTLS_DBNAME==NULL && _gnutls_db_func_is_ok(state)!=0) {
+	if (_gnutls_db_func_is_ok(state)!=0) {
 		return GNUTLS_E_DB_ERROR;
 	}
 	
@@ -317,13 +316,15 @@ int ret = 0;
 }
 
 /**
-  * gnutls_db_remove_session - This function will remove the current session data from the db
+  * gnutls_db_remove_session - This function will remove the current session data from the database
   * @state: is a &GNUTLS_STATE structure.
   *
   * This function will remove the current session data from the session
   * database. This will prevent future handshakes reusing these session
   * data. This function should be called if a session was terminated
-  * abnormaly.
+  * abnormaly, and before gnutls_deinit() is called.
+  *
+  * Normally gnutls_deinit() will remove abnormally terminated sessions.
   *
   **/
 void gnutls_db_remove_session(GNUTLS_STATE state) {
