@@ -717,6 +717,10 @@ int _gnutls_send_handshake(GNUTLS_STATE state, void *i_data,
 	uint32 datasize;
 	int pos = 0;
 
+	/* to know where the procedure was interrupted.
+	 */
+	state->gnutls_internals.handshake_direction = 1; /* write */
+
 	if (i_data == NULL && i_datasize == 0) {
 		/* we are resuming a previously interrupted
 		 * send.
@@ -960,6 +964,10 @@ int _gnutls_recv_handshake(GNUTLS_STATE state, uint8 ** data,
 	opaque *dataptr = NULL;
 	HandshakeType recv_type;
 
+	/* to know where the procedure was interrupted.
+	 */
+	state->gnutls_internals.handshake_direction = 0; /* read */
+
 	ret = _gnutls_recv_handshake_header(state, type, &recv_type);
 	if (ret < 0) {
 		if (ret == GNUTLS_E_UNEXPECTED_HANDSHAKE_PACKET
@@ -1055,6 +1063,8 @@ int _gnutls_recv_handshake(GNUTLS_STATE state, uint8 ** data,
 	return ret;
 }
 
+const static opaque EXPORT_CIPHERSUITE[2] = { 0x00, 0x03 };
+
 /* This function checks if the given cipher suite is supported, and sets it
  * to the state;
  */
@@ -1066,6 +1076,12 @@ static int _gnutls_client_set_ciphersuite(GNUTLS_STATE state,
 	uint16 x;
 	int i, err;
 
+	if (state->gnutls_internals.exportable_detection_hack != 0)
+		if ( memcmp( suite, EXPORT_CIPHERSUITE, 2)==0) {
+			gnutls_assert();
+			return GNUTLS_E_EXPORT_CIPHER_SUITE;
+		} 
+
 	z = 1;
 	x = _gnutls_supported_ciphersuites(state, &cipher_suites);
 	for (i = 0; i < x; i++) {
@@ -1294,6 +1310,7 @@ static int _gnutls_read_server_hello(GNUTLS_STATE state, char *data,
 	return ret;
 }
 
+
 /* This function copies the appropriate ciphersuites, to a localy allocated buffer 
  * Needed in client hello messages. Returns the new data length.
  */
@@ -1302,7 +1319,7 @@ static int _gnutls_copy_ciphersuites(GNUTLS_STATE state,
 {
 	int ret, i;
 	GNUTLS_CipherSuite *cipher_suites;
-	uint16 x;
+	uint16 cipher_num;
 	int datalen, pos;
 
 	ret = _gnutls_supported_ciphersuites_sorted(state, &cipher_suites);
@@ -1323,35 +1340,51 @@ static int _gnutls_copy_ciphersuites(GNUTLS_STATE state,
 		gnutls_assert();
 		return ret;
 	}
-	if (ret==0) {
+
+	/* If no cipher suites were enabled.
+	 */
+	if (ret == 0) {
 		gnutls_assert();
 		return GNUTLS_E_INSUFICIENT_CRED;
 	}
 
 
-	x = ret;
-	x *= sizeof(uint16);	/* in order to get bytes */
+	cipher_num = ret;
+	
+	/* for the EXPORT DETECTION */
+	if ( state->gnutls_internals.exportable_detection_hack != 0) {
+		cipher_num += 1; /* add 1 for the export cipher suite */
+	}
 
+	cipher_num *= sizeof(uint16);	/* in order to get bytes */
+ 
 	datalen = pos = 0;
 
-	datalen += sizeof(uint16) + x;
+	datalen += sizeof(uint16) + cipher_num;
 
 	*ret_data = gnutls_malloc(datalen);
-
-
 	if (*ret_data == NULL) {
 		gnutls_assert();
 		return GNUTLS_E_MEMORY_ERROR;
 	}
 
 
-	_gnutls_write_uint16(x, *ret_data);
+        /* add 2 for the export cipher suite 
+         */
+	_gnutls_write_uint16(cipher_num, *ret_data);
 	pos += 2;
 
-	for (i = 0; i < x / 2; i++) {
-		memcpy(&(*ret_data)[pos], cipher_suites[i].CipherSuite, 2);
+	for (i = 0; i < (cipher_num / 2) - 1; i++) {
+		memcpy( &(*ret_data)[pos], cipher_suites[i].CipherSuite, 2);
 		pos += 2;
 	}
+
+	/* for the EXPORT DETECTION */
+	if ( state->gnutls_internals.exportable_detection_hack != 0) {
+		memcpy( &(*ret_data)[pos], EXPORT_CIPHERSUITE, 2);
+		pos += 2;
+	}
+
 	gnutls_free(cipher_suites);
 
 	return datalen;
@@ -1365,7 +1398,7 @@ static int _gnutls_copy_comp_methods(GNUTLS_STATE state,
 				     opaque ** ret_data)
 {
 	int ret, i;
-	uint8 *compression_methods, z;
+	uint8 *compression_methods, comp_num;
 	int datalen, pos;
 
 	ret =
@@ -1376,10 +1409,10 @@ static int _gnutls_copy_comp_methods(GNUTLS_STATE state,
 		return ret;
 	}
 
-	z = ret;
+	comp_num = ret;
 
 	datalen = pos = 0;
-	datalen += z + 1;
+	datalen += comp_num + 1;
 
 	*ret_data = gnutls_malloc(datalen);
 	if (*ret_data == NULL) {
@@ -1387,9 +1420,9 @@ static int _gnutls_copy_comp_methods(GNUTLS_STATE state,
 		return GNUTLS_E_MEMORY_ERROR;
 	}
 
-	(*ret_data)[pos++] = z; /* put the number of compression methods */
+	(*ret_data)[pos++] = comp_num; /* put the number of compression methods */
 
-	for (i = 0; i < z; i++) {
+	for (i = 0; i < comp_num; i++) {
 		(*ret_data)[pos++] = compression_methods[i];
 	}
 
@@ -1429,9 +1462,7 @@ static int _gnutls_send_client_hello(GNUTLS_STATE state, int again)
 		/* 2 for version, (4 for unix time + 28 for random bytes==TLS_RANDOM_SIZE) 
 		 */
 		 
-		data = gnutls_malloc(datalen + 16);	/* 16 is added to avoid realloc
-							 * if no much data are added.
-							 */
+		data = gnutls_malloc(datalen);
 		if (data == NULL) {
 			gnutls_assert();
 			return GNUTLS_E_MEMORY_ERROR;
@@ -1757,6 +1788,22 @@ int gnutls_rehandshake(GNUTLS_STATE state)
 	return 0;
 }
 
+/**
+  * gnutls_handshake_get_direction - This function will return the state of the handshake protocol
+  * @state: is a a &GNUTLS_STATE structure.
+  *
+  * This function provides information about the handshake procedure, and
+  * is only useful if the gnutls_handshake() call was interrupted for some
+  * reason.
+  *
+  * Returns 0 if the function was interrupted while receiving data, and 
+  * 1 otherwise. 
+  *
+  **/
+int gnutls_handshake_get_direction(GNUTLS_STATE state) {
+	return state->gnutls_internals.handshake_direction;
+}
+
 static int _gnutls_abort_handshake( GNUTLS_STATE state, int ret) {
 	if ( ((ret==GNUTLS_E_WARNING_ALERT_RECEIVED) && 
 		( gnutls_alert_get(state) == GNUTLS_A_NO_RENEGOTIATION))
@@ -1999,6 +2046,10 @@ static int _gnutls_send_handshake_final(GNUTLS_STATE state, int init)
 {
 	int ret = 0;
 
+	/* to know where the procedure was interrupted.
+	 */
+	state->gnutls_internals.handshake_direction = 1; /* write */
+
 	/* Send the CHANGE CIPHER SPEC PACKET */
 
 	switch (STATE) {
@@ -2055,6 +2106,10 @@ static int _gnutls_recv_handshake_final(GNUTLS_STATE state, int init)
 	int ret = 0;
 	uint8 ch;
 
+	/* to know where the procedure was interrupted.
+	 */
+	state->gnutls_internals.handshake_direction = 0; /* recv */
+
 	switch (STATE) {
 	case STATE0:
 	case STATE30:
