@@ -561,6 +561,8 @@ typedef struct {
 	GNUTLS_Version		default_record_version;
 
 	int			cbc_protection_hack;
+	int			exportable_detection_hack;
+
 	int			rsa_pms_check; /* 0 means enabled */
 
 	void*			user_ptr;
@@ -569,6 +571,12 @@ typedef struct {
 						* enable cipher suites
 						* which have 0xFF status.
 						*/
+	
+	/* Holds 0 if the handshake procedure was interrupted while
+	 * receiving, and non zero otherwise.
+	 */
+	int			handshake_direction;
+
 	/* If you add anything here, check _gnutls_handshake_internal_state_clear().
 	 */
 } GNUTLS_INTERNALS;
