@@ -29,6 +29,7 @@
 
 #ifdef HAVE_LIBOPENCDK
 
+#include <gnutls_str.h>
 #include <stdio.h>
 #include <gcrypt.h>
 #include <opencdk.h>
@@ -42,6 +43,9 @@
 
 #define OPENPGP_NAME_SIZE GNUTLS_X509_CN_SIZE
 
+#define APPEND_DATUM(x, y, z) _gnutls_datum_append_m( x, y, z, realloc)
+#define RC(x) ((x) < 0)
+
 typedef struct {
     int type;
     int armored;
@@ -118,10 +122,10 @@ kbx_blob_release( keybox_blob *ctx )
     }
 } /* kbx_blob_release */
 
-static KEYDB_HD
+static CDK_KEYDB_HD
 kbx_to_keydb( keybox_blob *blob )
 {
-    KEYDB_HD khd = NULL;
+    CDK_KEYDB_HD khd = NULL;
 
     if ( !blob )
         return NULL;
@@ -220,14 +224,14 @@ kbnode_to_datum( CDK_KBNODE kb_pk, gnutls_datum *raw )
     CDK_KBNODE p = NULL;
     CDK_BSTRING dat = NULL;
     CDK_IOBUF a = NULL;
-    PACKET pkt = {0};
+    CDK_PACKET pkt = {0};
     int rc = 0;
   
     if ( !kb_pk || !raw )
         return GNUTLS_E_INVALID_PARAMETERS;
 
     /* fixme: conver the whole key */
-    for (p=kb_pk; p && p->pkt->pkttype; p=p->next) {
+    for ( p =kb_pk; p && p->pkt->pkttype; p = p->next ) {
         if (p->pkt->pkttype == PKT_PUBLIC_KEY) {
             a = cdk_iobuf_temp();
             pkt.pkttype = PKT_PUBLIC_KEY;
@@ -270,7 +274,7 @@ datum_to_kbnode( const gnutls_datum *raw, CDK_KBNODE *r_pkt )
     cdk_iobuf_write( buf, raw->data, raw->size );
     rc = cdk_keydb_get_keyblock( buf, &pkt, &dummy );
     if ( rc && rc != CDKERR_EOF ) {
-        rc = GNUTLS_E_NO_CERTIFICATE_FOUND;
+        rc = GNUTLS_E_UNKNOWN_ERROR;
         goto fail;
     }
     else
@@ -309,7 +313,7 @@ fail:
 }
 
 static int
-openpgp_pk_to_gnutls_cert(gnutls_cert *cert, PKT_public_key *pk)
+openpgp_pk_to_gnutls_cert( gnutls_cert *cert, cdkPKT_public_key *pk )
 {
     size_t nbytes = 0;
     int algo, i;
@@ -325,7 +329,6 @@ openpgp_pk_to_gnutls_cert(gnutls_cert *cert, PKT_public_key *pk)
     algo = is_DSA(pk->pubkey_algo)? GNUTLS_PK_DSA : GNUTLS_PK_RSA;
     cert->subject_pk_algorithm = algo;
     cert->version = pk->version;
-    cert->valid = 0; /* fixme: should set after the verification */
     cert->cert_type = GNUTLS_CRT_OPENPGP;
 
     if (is_DSA(pk->pubkey_algo) || pk->pubkey_algo == GCRY_PK_RSA_S)
@@ -338,9 +341,9 @@ openpgp_pk_to_gnutls_cert(gnutls_cert *cert, PKT_public_key *pk)
 
     cert->params_size = cdk_pk_get_npkey( pk->pubkey_algo );
     for (i=0; i<cert->params_size; i++) {      
-        nbytes = pk->mpi[i].bytes+2;
+        nbytes = pk->mpi[i]->bytes + 2;
         rc = _gnutls_mpi_scan_pgp( &cert->params[i], 
-                                   pk->mpi[i].data, &nbytes );
+                                   pk->mpi[i]->data, &nbytes );
         if ( rc ) {
             rc = GNUTLS_E_MPI_SCAN_FAILED;
             goto fail;
@@ -357,11 +360,11 @@ fail:
 }
 
 static int
-openpgp_sig_to_gnutls_cert(gnutls_cert *cert, PKT_signature *sig)
+openpgp_sig_to_gnutls_cert( gnutls_cert *cert, cdkPKT_signature *sig )
 {
     CDK_IOBUF buf = NULL;
     CDK_BSTRING a = NULL;
-    PACKET pkt;
+    CDK_PACKET pkt;
     size_t sigsize = 0;
     int rc = 0;
   
@@ -378,7 +381,7 @@ openpgp_sig_to_gnutls_cert(gnutls_cert *cert, PKT_signature *sig)
         goto fail;
     a = cdk_iobuf_read_mem( buf, 0 );
     if ( a ) {
-        rc = gnutls_datum_append( &cert->signature, a->d, a->len );
+        rc = APPEND_DATUM( &cert->signature, a->d, a->len );
         if (rc < 0) {
             gnutls_assert();
             rc = GNUTLS_E_MEMORY_ERROR;
@@ -409,7 +412,7 @@ _gnutls_openpgp_key2gnutls_key( gnutls_private_key *pkey,
 {
     CDK_KBNODE p = NULL, kb_sk;
     CDK_IOBUF buf;
-    PKT_secret_key *sk = NULL;
+    cdkPKT_secret_key *sk = NULL;
     int pke_algo, i, j, eof = 0;
     size_t nbytes = 0;
     int rc = 0;
@@ -428,16 +431,16 @@ _gnutls_openpgp_key2gnutls_key( gnutls_private_key *pkey,
     }
     p = cdk_kbnode_find( kb_sk, PKT_SECRET_KEY );
     if ( !p ) {
-        rc = GNUTLS_E_NO_CERTIFICATE_FOUND;
+        rc = GNUTLS_E_UNKNOWN_ERROR;
         goto leave;
     }
     sk = p->pkt->pkt.secret_key;
     pke_algo = sk->pk->pubkey_algo;
     pkey->params_size = cdk_pk_get_npkey( pke_algo );
     for ( i = 0; i < pkey->params_size; i++ ) {
-        nbytes = sk->pk->mpi[i].bytes+2;
+        nbytes = sk->pk->mpi[i]->bytes + 2;
         rc = _gnutls_mpi_scan_pgp( &pkey->params[i],
-                                   sk->pk->mpi[i].data, &nbytes );
+                                   sk->pk->mpi[i]->data, &nbytes );
         if ( rc ) {
             rc = GNUTLS_E_MPI_SCAN_FAILED;
             release_mpi_array( pkey->params, i-1 );
@@ -446,7 +449,7 @@ _gnutls_openpgp_key2gnutls_key( gnutls_private_key *pkey,
     }
     pkey->params_size += cdk_pk_get_nskey( pke_algo );
     for (j=0; j<cdk_pk_get_nskey( pke_algo ); j++, i++) {
-        nbytes = sk->mpi[j]->bytes+2;
+        nbytes = sk->mpi[j]->bytes + 2;
         rc = _gnutls_mpi_scan_pgp(&pkey->params[i],
                                   sk->mpi[j]->data, &nbytes);
         if ( rc ) {
@@ -486,7 +489,7 @@ int
 _gnutls_openpgp_cert2gnutls_cert(gnutls_cert *cert, gnutls_datum raw)
 {
     CDK_KBNODE p, kb_pk = NULL;
-    PKT_public_key *pk = NULL;
+    cdkPKT_public_key *pk = NULL;
     int rc = 0;
   
     if ( !cert )
@@ -498,7 +501,7 @@ _gnutls_openpgp_cert2gnutls_cert(gnutls_cert *cert, gnutls_datum raw)
         return rc;
     p = cdk_kbnode_find( kb_pk, PKT_PUBLIC_KEY );
     if ( !p ) {
-        rc = GNUTLS_E_NO_CERTIFICATE_FOUND;
+        rc = GNUTLS_E_UNKNOWN_ERROR;
         goto fail;
     }
     pk = p->pkt->pkt.public_key;
@@ -515,7 +518,7 @@ fail:
     return rc;
 }
 
-/**
+/*-
  * gnutls_openpgp_get_key - Retrieve a key from the keyring.
  * @key: the destination context to save the key.
  * @keyring: the datum struct that contains all keyring information.
@@ -524,15 +527,15 @@ fail:
  *
  * This function can be used to retrieve keys by different pattern
  * from a binary or a file keyring.
- **/
+ -*/
 int
 gnutls_openpgp_get_key(gnutls_datum *key, const gnutls_datum *keyring,
                        key_attr_t by, opaque *pattern)
 {
     keybox_blob *blob = NULL;
-    KEYDB_HD khd = NULL;
+    CDK_KEYDB_HD khd = NULL;
     CDK_KBNODE pk = NULL;
-    KEYDB_SEARCH ks;
+    CDK_KEYDB_SEARCH ks;
     int rc = 0;
   
     if ( !key || !keyring || by == KEY_ATTR_NONE )
@@ -563,12 +566,12 @@ gnutls_openpgp_get_key(gnutls_datum *key, const gnutls_datum *keyring,
   
     rc = cdk_keydb_search( khd, &ks, &pk );
     if ( rc ) {
-        rc = GNUTLS_E_NO_CERTIFICATE_FOUND;
+        rc = GNUTLS_E_UNKNOWN_ERROR;
         goto leave;
     }    
 
     if ( !cdk_kbnode_find( pk, PKT_PUBLIC_KEY ) ) {
-        rc = GNUTLS_E_NO_CERTIFICATE_FOUND;
+        rc = GNUTLS_E_UNKNOWN_ERROR;
         goto leave;
     }
 
@@ -625,7 +628,7 @@ gnutls_certificate_set_openpgp_key_mem( GNUTLS_CERTIFICATE_CREDENTIALS res,
             break;
         if (pkt->pkt->pkttype == PKT_PUBLIC_KEY) {
             int n = res->ncerts;
-            PKT_public_key *pk = pkt->pkt->pkt.public_key;
+            cdkPKT_public_key *pk = pkt->pkt->pkt.public_key;
             res->cert_list_length[n] = 1;
             gnutls_set_datum(&res->cert_list[n][0].raw,
                              cert->data, cert->size);
@@ -634,7 +637,7 @@ gnutls_certificate_set_openpgp_key_mem( GNUTLS_CERTIFICATE_CREDENTIALS res,
         }
         else if (pkt->pkt->pkttype == PKT_SIGNATURE) {
             int n = res->ncerts;
-            PKT_signature *sig = pkt->pkt->pkt.signature;
+            cdkPKT_signature *sig = pkt->pkt->pkt.signature;
             openpgp_sig_to_gnutls_cert( &res->cert_list[n][0], sig ); 
         }
     }
@@ -679,20 +682,28 @@ gnutls_certificate_set_openpgp_key_file(GNUTLS_CERTIFICATE_CREDENTIALS res,
     int eof = 0, i = 0;
     int rc = 0;
   
-    if ( !res || !KEYFILE || !CERTFILE )
+    if ( !res || !KEYFILE || !CERTFILE ) {
+    	gnutls_assert();
         return GNUTLS_E_INVALID_PARAMETERS;
-
-    if ( !file_exist(CERTFILE) || !file_exist(KEYFILE) )
+    }
+    
+    if ( !file_exist(CERTFILE) || !file_exist(KEYFILE) ) {
+        gnutls_assert();
         return GNUTLS_E_FILE_ERROR;
-  
+    }
+      
     rc = cdk_iobuf_open(&inp, CERTFILE, IOBUF_MODE_RD);
-    if ( rc )
+    if ( rc ) {
+        gnutls_assert();
         return GNUTLS_E_FILE_ERROR;
+    }
+    
     if ( cdk_armor_filter_use( inp ) ) {
         memset( &afx, 0, sizeof afx );
         rc = cdk_armor_filter( &afx, IOBUF_CTRL_UNDERFLOW, inp );
         if ( rc ) {
             cdk_iobuf_close( inp );
+            gnutls_assert();
             rc = GNUTLS_E_ASCII_ARMOR_ERROR;
             goto leave;          
         }
@@ -728,7 +739,7 @@ gnutls_certificate_set_openpgp_key_file(GNUTLS_CERTIFICATE_CREDENTIALS res,
                 break;
             if (pkt->pkt->pkttype == PKT_PUBLIC_KEY) {
                 int n = res->ncerts;
-                PKT_public_key *pk = pkt->pkt->pkt.public_key;
+                cdkPKT_public_key *pk = pkt->pkt->pkt.public_key;
                 res->cert_list_length[n] = 1;
                 iobuf_to_datum(inp, &res->cert_list[n][0].raw);
                 openpgp_pk_to_gnutls_cert( &res->cert_list[n][0], pk );
@@ -736,7 +747,7 @@ gnutls_certificate_set_openpgp_key_file(GNUTLS_CERTIFICATE_CREDENTIALS res,
             }
             else if (pkt->pkt->pkttype == PKT_SIGNATURE) {
                 int n = res->ncerts;
-                PKT_signature *sig = pkt->pkt->pkt.signature;
+                cdkPKT_signature *sig = pkt->pkt->pkt.signature;
                 openpgp_sig_to_gnutls_cert( &res->cert_list[n][0], sig );
             }
         }
@@ -745,19 +756,23 @@ gnutls_certificate_set_openpgp_key_file(GNUTLS_CERTIFICATE_CREDENTIALS res,
     cdk_iobuf_close(inp);
     if ( rc ) {
         cdk_kbnode_release( kb_pk );
+        gnutls_assert();
         rc = GNUTLS_E_UNKNOWN_ERROR;
         goto leave;
     }
     cdk_kbnode_release( kb_pk );
 
     rc = cdk_iobuf_open( &inp, KEYFILE, IOBUF_MODE_RD );
-    if ( rc )
+    if ( rc ) {
+        gnutls_assert();
         return GNUTLS_E_FILE_ERROR;
+    }
     if ( cdk_armor_filter_use( inp ) ) {      
         memset( &afx, 0, sizeof afx );
         rc = cdk_armor_filter( &afx, IOBUF_CTRL_UNDERFLOW, inp );
         if ( rc ) {
             cdk_iobuf_close( inp );
+            gnutls_assert();
             rc = GNUTLS_E_ASCII_ARMOR_ERROR;
             goto leave;
         }
@@ -816,7 +831,7 @@ gnutls_openpgp_extract_key_name( const gnutls_datum *cert,
                                  gnutls_openpgp_name *dn )
 {
     CDK_KBNODE kb_pk = NULL, pkt;
-    PKT_user_id *uid = NULL;
+    cdkPKT_user_id *uid = NULL;
     char *email;
     int pos = 0, pos1 = 0, pos2 = 0;
     size_t size = 0;
@@ -1002,27 +1017,27 @@ _gnutls_openpgp_get_key_trust(const char *trustdb,
 {
     CDK_KBNODE kb_pk = NULL, pkt;
     CDK_IOBUF buf;
-    PKT_public_key *pk = NULL;
+    cdkPKT_public_key *pk = NULL;
     int flags = 0, ot = 0, trustval = 0;
     int rc = 0;
 
     if ( !trustdb || !key || !r_success )
-        return GNUTLS_E_NO_CERTIFICATE_FOUND;
+        return GNUTLS_E_INVALID_REQUEST;
 
     *r_success = 0;
     rc = datum_to_kbnode( key, &kb_pk );
     if ( rc )
-        return GNUTLS_E_NO_CERTIFICATE_FOUND;
+        return GNUTLS_E_UNKNOWN_ERROR;
 
     pkt = cdk_kbnode_find( kb_pk, PKT_PUBLIC_KEY );
     if ( pkt )
         pk = pkt->pkt->pkt.public_key;
     if ( !pk )
-        return GNUTLS_E_NO_CERTIFICATE_FOUND;
+        return GNUTLS_E_UNKNOWN_ERROR;
 
     rc = cdk_iobuf_open( &buf, trustdb, IOBUF_MODE_RD );
     if ( rc ) {
-        trustval = GNUTLS_E_NO_CERTIFICATE_FOUND;
+        trustval = GNUTLS_E_FILE_ERROR;
         goto leave;
     }
     rc = cdk_trustdb_get_ownertrust( buf, pk, &ot, &flags );
@@ -1070,6 +1085,12 @@ leave:
  *
  * Verify all signatures in the certificate list. When the key
  * is not available, the signature is skipped.
+ *
+ * When the trustdb parameter is used, the function checks the
+ * ownertrust of the key before the signatures are checked. It
+ * is possible that the key was disabled or the owner is not trusted
+ * at all. Then we don't check the signatures because it makes no sense.
+ *
  * The return value is one of the CertificateStatus entries.
  **/
 int
@@ -1079,16 +1100,16 @@ gnutls_openpgp_verify_key( const char *trustdb,
                            int cert_list_length )
 {
     CDK_KBNODE kb_pk = NULL;
-    KEYDB_HD khd = NULL;
+    CDK_KEYDB_HD khd = NULL;
     keybox_blob *blob = NULL;
     int rc = 0;
     int status = 0;
   
-    if (!cert_list || !cert_list_length || !keyring)
+    if (!cert_list || cert_list_length != 1 || !keyring)
         return GNUTLS_E_NO_CERTIFICATE_FOUND;
 
-    if (cert_list_length != 1 || !keyring->size)
-        return GNUTLS_E_NO_CERTIFICATE_FOUND;
+    if ( !keyring->size && !trustdb)
+        return GNUTLS_E_INVALID_REQUEST;
 
     blob = kbx_read_blob(keyring, 0);
     if (!blob)
@@ -1148,7 +1169,7 @@ int
 gnutls_openpgp_fingerprint(const gnutls_datum *cert, char *fpr, size_t *fprlen)
 {
     CDK_KBNODE kb_pk = NULL, pkt;
-    PKT_public_key *pk = NULL;
+    cdkPKT_public_key *pk = NULL;
     int rc = 0;
   
     if (!cert || !fpr || !fprlen)
@@ -1172,25 +1193,26 @@ gnutls_openpgp_fingerprint(const gnutls_datum *cert, char *fpr, size_t *fprlen)
     return 0;
 }
 
-/**
+/*-
  * gnutls_openpgp_keyid - Gets the keyID
  * @cert: the raw data that contains the OpenPGP public key.
  * @keyid: the buffer to save the keyid.
  *
  * Returns the 64-bit keyID of the OpenPGP key.
- **/
+ -*/
 int
-gnutls_openpgp_keyid( const gnutls_datum *cert, uint32 *keyid )
+gnutls_openpgp_keyid( const gnutls_datum *cert, unsigned char keyid[8] )
 {
     CDK_KBNODE kb_pk = NULL, pkt;
-    PKT_public_key *pk = NULL;
+    cdkPKT_public_key *pk = NULL;
+    u32 kid[2];
     int rc = 0;
   
     if (!cert || !keyid)
         return GNUTLS_E_UNKNOWN_ERROR;
 
     rc = datum_to_kbnode( cert, &kb_pk );
-    if (rc)
+    if ( rc )
         return rc;
 
     pkt = cdk_kbnode_find( kb_pk, PKT_PUBLIC_KEY );
@@ -1198,12 +1220,16 @@ gnutls_openpgp_keyid( const gnutls_datum *cert, uint32 *keyid )
         pk = pkt->pkt->pkt.public_key;
     if ( !pk )
         return GNUTLS_E_UNKNOWN_ERROR;
-    cdk_pk_get_keyid( pk, (u32 *)keyid );
+    cdk_pk_get_keyid( pk, kid );
+    keyid[0] = kid[0] >> 24; keyid[1] = kid[0] >> 16;
+    keyid[2] = kid[0] >>  8; keyid[3] = kid[0];
+    keyid[4] = kid[1] >> 24; keyid[5] = kid[1] >> 16;
+    keyid[6] = kid[1] >>  8; keyid[7] = kid[1];
   
     return 0;
 }
 
-/**
+/*-
  * gnutls_openpgp_add_keyring_file - Adds a keyring file for OpenPGP
  * @keyring: data buffer to store the file.
  * @name: filename of the keyring.
@@ -1211,7 +1237,7 @@ gnutls_openpgp_keyid( const gnutls_datum *cert, uint32 *keyid )
  * The function is used to set keyrings that will be used internally
  * by various OpenCDK functions. For example to find a key when it
  * is needed for an operations.
- **/
+ -*/
 int
 gnutls_openpgp_add_keyring_file(gnutls_datum *keyring, const char *name)
 {
@@ -1233,7 +1259,38 @@ gnutls_openpgp_add_keyring_file(gnutls_datum *keyring, const char *name)
     blob = kbx_data_to_keyring( KBX_BLOB_FILE, enc, name,
                                 strlen(name), &nbytes);
     if ( blob && nbytes ) { 
-        if ( gnutls_datum_append( keyring, blob, nbytes ) < 0 ) {
+        if ( APPEND_DATUM( keyring, blob, nbytes ) < 0 ) {
+            gnutls_assert();
+            return GNUTLS_E_MEMORY_ERROR;
+        }
+        gnutls_free(blob);
+    }
+  
+    return 0;
+}
+
+/*-
+ * gnutls_openpgp_add_keyring_mem - Adds keyring data for OpenPGP
+ * @keyring: data buffer to store the file.
+ * @data: the binary data of the keyring.
+ * @len: the size of the binary buffer.
+ *
+ * Same as gnutls_openpgp_add_keyring_mem but now we store the
+ * data instead of the filename.
+ -*/
+int
+gnutls_openpgp_add_keyring_mem(gnutls_datum *keyring,
+                               const opaque *data, size_t len)
+{
+    uint8 *blob;
+    size_t nbytes = 0;
+  
+    if (!keyring || !data || !len)
+        return GNUTLS_E_INVALID_PARAMETERS;
+  
+    blob = kbx_data_to_keyring( KBX_BLOB_DATA, 0, data, len, &nbytes );
+    if ( blob && nbytes ) {
+        if ( APPEND_DATUM( keyring, blob, nbytes ) < 0 ) {
             gnutls_assert();
             return GNUTLS_E_MEMORY_ERROR;
         }
@@ -1244,36 +1301,15 @@ gnutls_openpgp_add_keyring_file(gnutls_datum *keyring, const char *name)
 }
 
 /**
- * gnutls_openpgp_add_keyring_mem - Adds keyring data for OpenPGP
- * @keyring: data buffer to store the file.
- * @data: the binary data of the keyring.
- * @len: the size of the binary buffer.
+ * gnutls_certificate_set_openpgp_keyring_file - Adds a keyring file for OpenPGP * @c: A certificate credentials structure
+ * @file: filename of the keyring.
+ *
+ * The function is used to set keyrings that will be used internally
+ * by various OpenPGP functions. For example to find a key when it
+ * is needed for an operations. The keyring will also be used at the
+ * verification functions.
  *
- * Same as gnutls_openpgp_add_keyring_mem but now we store the
- * data instead of the filename.
  **/
-int
-gnutls_openpgp_add_keyring_mem(gnutls_datum *keyring,
-                               const char *data, size_t len)
-{
-    uint8 *blob;
-    size_t nbytes = 0;
-  
-    if (!keyring || !data || !len)
-        return GNUTLS_E_INVALID_PARAMETERS;
-  
-    blob = kbx_data_to_keyring( KBX_BLOB_DATA, 0, data, len, &nbytes );
-    if ( blob && nbytes ) {
-        if ( gnutls_datum_append( keyring, blob, nbytes ) < 0 ) {
-            gnutls_assert();
-            return GNUTLS_E_MEMORY_ERROR;
-        }
-        gnutls_free(blob);
-    }
-  
-    return 0;
-}
-
 int
 gnutls_certificate_set_openpgp_keyring_file(GNUTLS_CERTIFICATE_CREDENTIALS c,
                                             const char *file)
@@ -1288,45 +1324,50 @@ gnutls_certificate_set_openpgp_keyring_file(GNUTLS_CERTIFICATE_CREDENTIALS c,
 }
 
 int
-gnutls_certificate_set_openpgp_keyring_mem(GNUTLS_CERTIFICATE_CREDENTIALS c,
-                                           const char *file)
+gnutls_certificate_set_openpgp_keyring_mem( GNUTLS_CERTIFICATE_CREDENTIALS c,
+                                            const opaque *data, size_t dlen )
 {
-    CDK_IOBUF inp = NULL;
+    CDK_IOBUF out = NULL;
     CDK_BSTRING a = NULL;
     armor_filter_s afx;
     int rc = 0;
   
-    if (!c || !file)
+    if ( !c || !data || !dlen )
         return GNUTLS_E_INVALID_PARAMETERS;
 
-    if ( !file_exist(file) )
-        return GNUTLS_E_FILE_ERROR;
-
-    rc = cdk_iobuf_open(&inp, file, IOBUF_MODE_RD);
+    rc = cdk_iobuf_create( &out, NULL );
     if ( rc )
         return GNUTLS_E_FILE_ERROR;
-    if ( cdk_armor_filter_use( inp ) ) {
+    rc = cdk_iobuf_write( out, data, dlen );
+    if ( rc ) {
+        cdk_iobuf_close( out );
+        return GNUTLS_E_FILE_ERROR;
+    }
+    /* Maybe it's a little confusing that we check the output..
+       but it's possible, that the data we want to add, is armored
+       and we only want to store plaintext keyring data. */
+    if ( cdk_armor_filter_use( out ) ) {
         memset( &afx, 0, sizeof afx );
-        rc = cdk_armor_filter( &afx, IOBUF_CTRL_UNDERFLOW, inp );
+        rc = cdk_armor_filter( &afx, IOBUF_CTRL_UNDERFLOW, out );
         if ( rc ) {
-            cdk_iobuf_close( inp );
+            cdk_iobuf_close( out );
             return GNUTLS_E_ASCII_ARMOR_ERROR;
         }
     }
 
-    a = cdk_iobuf_read_mem( inp, 0 );
+    a = cdk_iobuf_read_mem( out, 0 );
     if ( a ) {
         rc = gnutls_openpgp_add_keyring_mem( &c->keyring, a->d, a->len );
         cdk_free( a );
     }
     else
         rc = GNUTLS_E_UNKNOWN_ERROR;
-    cdk_iobuf_close( inp );
+    cdk_iobuf_close( out );
   
     return rc;
 }
 
-/**
+/*-
  * gnutls_openpgp_recv_key - Receives a key from a HKP keyserver.
  * @host - the hostname of the keyserver.
  * @port - the service port (if not set use 11371).
@@ -1334,7 +1375,7 @@ gnutls_certificate_set_openpgp_keyring_mem(GNUTLS_CERTIFICATE_CREDENTIALS c,
  * @key - Context to store the raw (dearmored) key.
  *
  * Try to connect to a public keyserver to get the specified key.
- **/
+ -*/
 int
 gnutls_openpgp_recv_key(const char *host, short port, uint32 keyid,
                         gnutls_datum *key)
@@ -1458,6 +1499,7 @@ _gnutls_openpgp_request_key( gnutls_datum* ret,
     if ( rc >= 0 )
         goto leave;
 
+    /* fixme: we can't use the code for old V3 RSA keys! */
     keyid = buffer_to_u32( key_fpr + (key_fpr_size-4) );
     rc = gnutls_openpgp_recv_key( cred->pgp_key_server,
                                   cred->pgp_key_server_port,
@@ -1479,29 +1521,407 @@ leave:
  * process.
  *
  **/
-void
+int
 gnutls_certificate_set_openpgp_keyserver(GNUTLS_CERTIFICATE_CREDENTIALS res,
                                          char* keyserver,
                                          int port)
 {
     if ( !res || !keyserver )
-        return;
+        return GNUTLS_E_ILLEGAL_PARAMETER;
 
     if (!port)
         port = 11371;
   
+    gnutls_free( res->pgp_key_server);
     res->pgp_key_server = gnutls_strdup( keyserver );
+    if ( res->pgp_key_server == NULL)
+    	return GNUTLS_E_MEMORY_ERROR;
+
     res->pgp_key_server_port = port;
+
+   return 0;
 }
 
-void
-gnutls_certificate_set_openpgp_trustdb(GNUTLS_CERTIFICATE_CREDENTIALS res,
-                                       char* trustdb)
+static int
+xml_add_tag( gnutls_string *xmlkey, const char *tag, const char *val )
+{
+    int rc = 0;
+
+    if ( !xmlkey || !tag || !val )
+        return GNUTLS_E_INVALID_PARAMETERS;
+    
+    rc = _gnutls_string_append_str( xmlkey, "    <" );
+    if ( !RC( rc ) )
+        rc = _gnutls_string_append_str( xmlkey, tag );
+    if ( !RC( rc ) )
+        rc = _gnutls_string_append_str( xmlkey, ">" );
+    if ( !RC( rc ) )
+        rc = _gnutls_string_append_str( xmlkey, val );
+    if ( !RC( rc ) )
+        rc =_gnutls_string_append_str( xmlkey, "</" );
+    if ( !RC( rc ) )
+        rc = _gnutls_string_append_str( xmlkey, tag );
+    if ( !RC( rc ) )
+        rc =_gnutls_string_append_str( xmlkey, ">\n" );
+    if ( !RC( rc ) )
+        rc = 0;
+    return rc;
+}
+
+static int
+xml_add_mpi( gnutls_string *xmlkey, CDK_MPI *m, const char *tag )
+{
+    char *p = NULL;
+    int i = 0, rc = 0;
+
+    if ( !xmlkey || !m || !tag )
+        return GNUTLS_E_INVALID_PARAMETERS;
+        
+    p = gnutls_calloc( 1, 2 * ( m->bytes + 3 ) );
+    for ( i = 0; i < (m->bytes + 2); i++ )
+        sprintf( p + 2 * i, "%02X", m->data[i] );
+    p[2 * ( m->bytes + 2 )] = '\0';
+    
+    rc = xml_add_tag( xmlkey, tag, p );
+    gnutls_free( p );
+    if ( rc < 0 ) return rc;
+    
+    return 0;
+}
+
+static int
+xml_add_key_mpi( gnutls_string *xmlkey, cdkPKT_public_key *pk )
+{
+    const char *s = "    <KEY ENCODING=\"HEX\"/>\n";
+    int rc = 0;
+
+    if ( !xmlkey || !pk )
+        return GNUTLS_E_INVALID_PARAMETERS;
+    
+    rc =_gnutls_string_append_str( xmlkey, s );
+    if ( rc < 0 ) return rc;
+         
+    if ( is_RSA( pk->pubkey_algo ) ) {
+        rc = xml_add_mpi( xmlkey, pk->mpi[0], "RSA-N" );
+        if ( rc < 0 ) return rc;
+        rc = xml_add_mpi( xmlkey, pk->mpi[1], "RSA-E" );
+        if ( rc < 0 ) return rc;
+    }
+    else if ( is_DSA( pk->pubkey_algo ) ) {
+        rc = xml_add_mpi( xmlkey, pk->mpi[0], "DSA-P" );
+        if ( rc < 0 ) return rc;
+        
+        rc = xml_add_mpi( xmlkey, pk->mpi[1], "DSA-Q" );
+        if ( rc < 0 ) return rc;
+        
+        rc = xml_add_mpi( xmlkey, pk->mpi[2], "DSA-G" );
+        if ( rc < 0 ) return rc;
+
+        rc = xml_add_mpi( xmlkey, pk->mpi[3], "DSA-Y" );
+        if ( rc < 0 ) return rc;
+    }
+    else if ( is_ELG( pk->pubkey_algo ) ) {
+        rc = xml_add_mpi( xmlkey, pk->mpi[0], "ELG-P" );
+        if ( rc < 0 ) return rc;
+
+        rc = xml_add_mpi( xmlkey, pk->mpi[1], "ELG-G" );
+        if ( rc < 0 ) return rc;
+
+        rc = xml_add_mpi( xmlkey, pk->mpi[2], "ELG-Y" );
+        if ( rc < 0 ) return rc;
+    }
+    else
+        return GNUTLS_E_UNWANTED_ALGORITHM;
+    
+    return 0;
+}
+
+static int
+xml_add_key( gnutls_string *xmlkey, int ext, cdkPKT_public_key *pk, int sub )
+{
+    const char *algo, *s;
+    char keyid[16], fpr[41], tmp[32];
+    byte fingerpr[20];
+    u32 kid[2];
+    int i = 0, rc = 0;
+
+    if ( !xmlkey || !pk )
+        return GNUTLS_E_INVALID_PARAMETERS;
+        
+    s = sub? "  <SUBKEY>\n" : "  <MAINKEY>\n";
+    rc = _gnutls_string_append_str( xmlkey, s );
+    if ( rc < 0 ) return rc;
+
+    cdk_pk_get_keyid( pk, kid );
+    snprintf( keyid, 16, "%08X%08X", kid[0], kid[1] );
+    rc = xml_add_tag( xmlkey, "KEYID", keyid );
+    if ( rc < 0 ) return rc;
+
+    cdk_pk_get_fingerprint( pk, fingerpr );
+    for ( i = 0; i < 20; i++ )
+        sprintf( fpr + 2 * i, "%02X", fingerpr[i] );
+    fpr[40] = '\0';
+    rc = xml_add_tag( xmlkey, "FINGERPRINT", fpr );
+    if ( rc < 0 ) return rc;
+
+    if ( is_DSA( pk->pubkey_algo ) ) algo = "DSA";
+    else if ( is_RSA( pk->pubkey_algo ) ) algo = "RSA";
+    else if ( is_ELG( pk->pubkey_algo ) ) algo = "ELG";
+    else algo = "???";
+    rc = xml_add_tag( xmlkey, "PKALGO", algo );
+    if ( rc < 0 ) return rc;
+
+    sprintf( tmp, "%d", cdk_pk_get_nbits( pk ) );
+    rc = xml_add_tag( xmlkey, "KEYLEN", tmp );
+    if ( rc < 0 ) return rc;
+
+    sprintf( tmp, "%u", pk->timestamp );
+    rc = xml_add_tag( xmlkey, "CREATED", tmp );
+    if ( rc < 0 ) return rc;
+
+    if ( pk->expiredate > 0 ) {
+        sprintf( tmp, "%u", pk->expiredate );
+        rc = xml_add_tag( xmlkey, "EXPIREDATE", tmp );
+	if ( rc < 0) return rc;
+    }
+
+    sprintf( tmp, "%d", pk->is_revoked );
+    rc = xml_add_tag( xmlkey, "REVOKED", tmp );
+    if ( rc < 0 ) return rc;
+
+    if ( ext ) {
+        rc = xml_add_key_mpi( xmlkey, pk );
+        if ( rc < 0 ) return rc;
+    }
+     
+    s = sub? "  </SUBKEY>\n" : "  </MAINKEY>\n";
+    
+    rc = _gnutls_string_append_str( xmlkey, s );
+    if ( rc < 0 ) return rc;
+
+    return 0;
+}
+
+static int
+xml_add_userid( gnutls_string *xmlkey, int ext,
+                gnutls_openpgp_name *dn, cdkPKT_user_id *id )
+{
+    const char *s;
+    char *p, *name, tmp[32];
+    int rc = 0;
+
+    if ( !xmlkey || !dn || !id )
+        return GNUTLS_E_INVALID_PARAMETERS;
+
+    s = "  <USERID>\n";
+    rc = _gnutls_string_append_str( xmlkey, s );
+    if ( rc < 0 ) return rc;
+
+    p = strchr( dn->name, '<' );
+    if ( p ) {
+        int len = (p - dn->name - 1);
+        name = gnutls_calloc( 1, len );
+        if ( name == NULL ) return GNUTLS_E_MEMORY_ERROR;
+        memcpy( name, dn->name, len );
+
+        rc = xml_add_tag( xmlkey, "NAME", name );
+        gnutls_free( name );
+        if ( rc < 0 ) return rc;
+    }
+    else {
+        rc = xml_add_tag( xmlkey, "NAME", dn->name );
+        if ( rc < 0 ) return rc;
+    }
+    
+    rc = xml_add_tag( xmlkey, "EMAIL", dn->email );
+    if ( rc < 0 ) return rc;
+
+    if ( ext ) {
+        sprintf( tmp, "%d", id->is_primary );
+        rc = xml_add_tag( xmlkey, "PRIMARY", tmp );
+        if ( rc < 0 ) return rc;
+        
+        sprintf( tmp, "%d", id->is_revoked );
+        rc = xml_add_tag( xmlkey, "REVOKED", tmp );
+        if ( rc < 0 ) return rc;
+    }
+
+    s = "  </USERID>\n";
+    rc = _gnutls_string_append_str( xmlkey, s );
+    if ( rc < 0 ) return rc;
+    
+    return 0;
+}
+
+static int
+xml_add_sig( gnutls_string *xmlkey, int ext, cdkPKT_signature *sig )
+{
+    const char *algo, *s;
+    char tmp[32], keyid[16];
+    u32 kid[2];
+    int rc = 0;
+
+    if ( !xmlkey || !sig )
+        return GNUTLS_E_INVALID_PARAMETERS;
+
+    s = "  <SIGNATURE>\n";
+    rc = _gnutls_string_append_str( xmlkey, s );
+    if ( rc < 0 ) return rc;
+
+    sprintf( tmp, "%d", sig->version );
+    rc = xml_add_tag( xmlkey, "VERSION", tmp );
+    if ( rc < 0 ) return rc;
+
+    if ( ext ) {
+        sprintf( tmp, "%d", sig->sig_class );
+        rc = xml_add_tag( xmlkey, "SIGCLASS", tmp );
+        if ( rc < 0 ) return rc;
+    }
+    
+    sprintf( tmp, "%d", sig->flags.expired );
+    rc = xml_add_tag( xmlkey, "EXPIRED", tmp );
+    if ( rc < 0 ) return rc;
+
+    if ( ext ) {
+        if ( is_DSA( sig->pubkey_algo ) ) algo = "DSA";
+        else if ( is_ELG( sig->pubkey_algo ) ) algo = "ELG";
+        else if ( is_RSA( sig->pubkey_algo ) ) algo = "RSA";
+        else algo = "???"; /* unknown algorithm */
+        rc = xml_add_tag( xmlkey, "PKALGO", algo );
+        if ( rc < 0 ) return rc;
+
+        if ( sig->digest_algo == GCRY_MD_SHA1 ) algo = "SHA1";
+        else if ( sig->digest_algo == GCRY_MD_RMD160 ) algo = "RMD160";
+        else if ( sig->digest_algo == GCRY_MD_MD5 ) algo = "MD5";
+        else algo = "???"; /* unknown algorithm */
+        rc = xml_add_tag( xmlkey, "MDALGO", algo );
+        if ( rc < 0 ) return rc;
+    }    
+
+    sprintf( tmp, "%u", sig->timestamp );
+    rc = xml_add_tag( xmlkey, "CREATED", tmp );
+    if ( rc < 0 ) return rc;
+
+    cdk_sig_get_keyid( sig, kid );
+    snprintf( keyid, 16, "%08X%08X", kid[0], kid[1] );
+    rc = xml_add_tag( xmlkey, "KEYID", keyid );
+    if ( rc < 0 ) return rc;
+
+    s = "  </SIGNATURE>\n";
+
+    rc = _gnutls_string_append_str( xmlkey, s );
+    if ( rc < 0 ) return rc;
+
+    return 0;
+}
+
+/**
+ * gnutls_openpgp_key_to_xml - Return a certificate as a XML fragment
+ * @cert: the certificate which holds the whole OpenPGP key.
+ * @xmlkey: he datum struct to store the XML result.
+ * @ext: extension mode (1/0), 1 means include key signatures and key data.
+ *
+ * This function will return the all OpenPGP key information encapsulated as
+ * a XML string.
+ **/
+int
+gnutls_openpgp_key_to_xml( const gnutls_datum *cert,
+                            gnutls_datum *xmlkey, int ext )
+{
+    CDK_KBNODE kb_pk, p;
+    CDK_PACKET *pkt;
+    gnutls_openpgp_name dn;
+    const char *s;
+    int idx = 0, rc = 0;
+    gnutls_string string_xml_key;
+
+    if ( !cert || !xmlkey )
+        return GNUTLS_E_INVALID_PARAMETERS;
+        
+    rc = datum_to_kbnode( cert, &kb_pk );
+    if ( rc )
+        return rc;
+
+    _gnutls_string_init( &string_xml_key, malloc, realloc, free);
+    memset( xmlkey, 0, sizeof *xmlkey );
+
+    s = "<?xml version=\"1.0\"?>\n\n";
+    rc = _gnutls_string_append_str( &string_xml_key, s );
+    if ( rc < 0 ) return rc;
+    
+    s = "<OPENPGPKEY>\n";
+    rc = _gnutls_string_append_str( &string_xml_key, s );
+    if ( rc < 0 ) return rc;
+
+    for ( p = kb_pk; p; p = p->next ) {
+        pkt = p->pkt;
+        switch ( pkt->pkttype ) {
+        case PKT_PUBLIC_KEY:
+            rc = xml_add_key( &string_xml_key, ext, pkt->pkt.public_key, 0 );
+            if ( rc < 0) return rc;
+            break;
+
+        case PKT_PUBLIC_SUBKEY:
+            rc = xml_add_key( &string_xml_key, ext, pkt->pkt.public_key, 1 );
+            if ( rc < 0 ) return rc;
+            break;
+
+        case PKT_USER_ID:
+            gnutls_openpgp_extract_key_name( cert, idx, &dn );
+            rc = xml_add_userid( &string_xml_key, ext, &dn, pkt->pkt.user_id );
+            if ( rc < 0) return rc;
+            idx++;
+            break;
+
+        case PKT_SIGNATURE:
+            rc = xml_add_sig( &string_xml_key, ext, pkt->pkt.signature );
+            if ( rc < 0 ) return rc;
+            break;
+            
+        default:
+            break;
+        }
+    }
+    s = "</OPENPGPKEY>\n";
+    rc = _gnutls_string_append_str( &string_xml_key, s );
+    if ( rc < 0 ) return rc;
+
+    rc = _gnutls_string_append_data( &string_xml_key, "\n\0", 2);
+    if ( rc < 0 ) return rc;
+    
+    *xmlkey = _gnutls_string2datum( &string_xml_key);
+
+    return rc;
+}   
+
+/**
+ * gnutls_certificate_set_openpgp_trustdb - Used to set an GnuPG trustdb
+ * @res: the destination context to save the data.
+ * @trustdb: is the trustdb filename
+ *
+ * This funtion will set a GnuPG trustdb which will be used in key
+ * verification functions. Only version 3 trustdb files are supported.
+ *
+ **/
+int
+gnutls_certificate_set_openpgp_trustdb( GNUTLS_CERTIFICATE_CREDENTIALS res,
+                                        char* trustdb )
 {
     if ( !res || !trustdb )
-        return;
+        return GNUTLS_E_ILLEGAL_PARAMETER;
 
-    res->pgp_trustdb = gnutls_strdup(trustdb);
+    if ( cdk_trustdb_check( trustdb, 3 ) ) {
+        /* The trustdb version is less then 3 and this mean the old
+           format is still used. We don't support this format. */
+        return GNUTLS_E_OPENPGP_TRUSTDB_VERSION_UNSUPPORTED;
+    }
+
+    gnutls_free( res->pgp_trustdb);
+    res->pgp_trustdb = gnutls_strdup( trustdb );
+    if ( res->pgp_trustdb==NULL )
+        return GNUTLS_E_MEMORY_ERROR;
+        
+    return 0;
 } 
 
 #else /*!HAVE_LIBOPENCDK*/
@@ -1528,9 +1948,9 @@ gnutls_certificate_set_openpgp_key_mem(GNUTLS_CERTIFICATE_CREDENTIALS res,
 }
 
 int
-gnutls_certificate_set_openpgp_key_file(GNUTLS_CERTIFICATE_CREDENTIALS res,
-                                        char* CERTFILE,
-                                        char* KEYFILE)
+gnutls_certificate_set_openpgp_key_file( GNUTLS_CERTIFICATE_CREDENTIALS res,
+                                         char* CERTFILE,
+                                         char* KEYFILE )
 {
     return GNUTLS_E_UNIMPLEMENTED_FEATURE;
 }
@@ -1581,28 +2001,29 @@ gnutls_openpgp_fingerprint(const gnutls_datum *cert, char *fpr, size_t *fprlen)
 }
 
 int
-gnutls_openpgp_add_keyring_file(gnutls_datum *keyring, const char *name)
+gnutls_openpgp_add_keyring_file( gnutls_datum *keyring, const char *name )
 {
     return GNUTLS_E_UNIMPLEMENTED_FEATURE; 
 }
 
 int
-gnutls_openpgp_add_keyring_mem(gnutls_datum *keyring,
-                               const char *data, size_t len)
+gnutls_openpgp_add_keyring_mem( gnutls_datum *keyring,
+                                const opaque *data, size_t len )
 {
     return GNUTLS_E_UNIMPLEMENTED_FEATURE; 
 }
 
 int
-gnutls_certificate_set_openpgp_keyring_file(GNUTLS_CERTIFICATE_CREDENTIALS c,
-                                            const char *file)
+gnutls_certificate_set_openpgp_keyring_file( GNUTLS_CERTIFICATE_CREDENTIALS c,
+                                             const char *file )
 {
     return GNUTLS_E_UNIMPLEMENTED_FEATURE;
 }
 
 int
-gnutls_certificate_set_openpgp_keyring_mem(GNUTLS_CERTIFICATE_CREDENTIALS c,
-                                           const char *file)
+gnutls_certificate_set_openpgp_keyring_mem( GNUTLS_CERTIFICATE_CREDENTIALS c,
+						const opaque* data,
+                                            size_t dlen)
 {
     return GNUTLS_E_UNIMPLEMENTED_FEATURE; 
 }
@@ -1611,26 +2032,32 @@ int
 _gnutls_openpgp_request_key( gnutls_datum* ret,
                              const GNUTLS_CERTIFICATE_CREDENTIALS cred,
                              opaque* key_fpr,
-                             int key_fpr_size)
+                             int key_fpr_size )
 {
     return GNUTLS_E_UNIMPLEMENTED_FEATURE; 
 }
 
-void
-gnutls_certificate_set_openpgp_keyserver(GNUTLS_CERTIFICATE_CREDENTIALS res,
-                                         char* keyserver,
-                                         int port)
+int
+gnutls_certificate_set_openpgp_keyserver( GNUTLS_CERTIFICATE_CREDENTIALS res,
+                                          char* keyserver,
+                                          int port )
 {
-    return;
+    return GNUTLS_E_UNIMPLEMENTED_FEATURE;
 }
 
-void
-gnutls_certificate_set_openpgp_trustdb(GNUTLS_CERTIFICATE_CREDENTIALS res,
-                                       char* trustdb)
+int
+gnutls_certificate_set_openpgp_trustdb( GNUTLS_CERTIFICATE_CREDENTIALS res,
+                                        char* trustdb )
 {
-    return;
-}     
+    return GNUTLS_E_UNIMPLEMENTED_FEATURE;
+}
 
+int
+gnutls_openpgp_key_to_xml( const gnutls_datum *cert,
+                            gnutls_datum *xmlkey, int ext )
+{
+    return GNUTLS_E_UNIMPLEMENTED_FEATURE;   
+}
 #endif /* HAVE_LIBOPENCDK */
 
 
