@@ -1,6 +1,6 @@
 #include <stdio.h>
-#include "../lib/gnutls.h"
-#include "../libextra/gnutls-extra.h"
+#include <gnutls/gnutls.h>
+#include <gnutls/extra.h>
 #include <time.h>
 
 #define PRINTX(x,y) if (y[0]!=0) printf(" #   %s %s\n", x, y)
@@ -47,7 +47,6 @@ void print_x509_info(GNUTLS_STATE state)
 	if (cert_list_size <= 0)
 		return;
 
-
 	printf(" - Certificate info:\n");
 
 	printf(" # Certificate is valid since: %s", my_ctime( &activet));
@@ -125,7 +124,11 @@ void print_openpgp_info(GNUTLS_STATE state)
 		int algo, bits;
 
 		printf(" # Key was created at: %s", my_ctime( &activet));
-		printf(" # Key expires: %s", my_ctime( &expiret));
+		printf(" # Key expires: ");
+		if (expiret != 0)
+			printf("%s", my_ctime( &expiret));
+		else
+			printf("Never\n");
 		
 		if (gnutls_openpgp_fingerprint
 		    (&cert_list[0], digest, &digest_size) >= 0) {
@@ -305,3 +308,20 @@ void print_list(void)
 
 	return;
 }
+
+void print_license(void)
+{
+   fprintf(stdout,
+	   "\nCopyright (C) 2001-2002 Nikos Mavroyanopoulos\n"
+	     "This program is free software; you can redistribute it and/or modify \n"
+	     "it under the terms of the GNU General Public License as published by \n"
+	     "the Free Software Foundation; either version 2 of the License, or \n"
+	     "(at your option) any later version. \n" "\n"
+	     "This program is distributed in the hope that it will be useful, \n"
+	     "but WITHOUT ANY WARRANTY; without even the implied warranty of \n"
+	     "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the \n"
+	     "GNU General Public License for more details. \n" "\n"
+	     "You should have received a copy of the GNU General Public License \n"
+	     "along with this program; if not, write to the Free Software \n"
+	     "Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n\n");
+}
