@@ -455,7 +455,7 @@ int main(int argc, char **argv)
    }
 
    if (gnutls_global_init_extra() < 0) {
-      fprintf(stderr, "global state initialization error\n");
+      fprintf(stderr, "global state (extra) initialization error\n");
       exit(1);
    }
 
@@ -618,7 +618,7 @@ int main(int argc, char **argv)
 			ctt[strlen(ctt)-1] = 0;
 
 /*
-		        printf("- connection from %s, port %d\n",
+		        printf("\n* connection from %s, port %d\n",
 			     inet_ntop(AF_INET, &client_address.sin_addr, topbuf,
 			       sizeof(topbuf)), ntohs(client_address.sin_port));
       */
@@ -651,7 +651,7 @@ int main(int argc, char **argv)
         	 	  printf("*** This is a resumed session\n");
 
 			if (quiet == 0) {
-			      printf("- connection from %s, port %d\n",
+			      printf("\n* connection from %s, port %d\n",
 				     inet_ntop(AF_INET, &client_address.sin_addr, topbuf,
 				       sizeof(topbuf)), ntohs(client_address.sin_port));
 			      print_info( j->tstate);
@@ -681,6 +681,8 @@ int main(int argc, char **argv)
 			
 		    }
 /* check if we have a full HTTP header */
+
+ 	            j->http_response = NULL;
 		    if (j->http_request!=NULL) {
 		        if ( (http==0 && strchr(j->http_request, '\n')) || strstr (j->http_request, "\r\n\r\n")) {
 			  get_response (j->tstate, j->http_request, &j->http_response, &j->response_length);
@@ -898,6 +900,8 @@ void gaa_parser(int argc, char **argv)
 	    cipher_priority[j++] = GNUTLS_CIPHER_3DES_CBC;
 	 if (strncasecmp(info.ciphers[i], "ARC", 3) == 0)
 	    cipher_priority[j++] = GNUTLS_CIPHER_ARCFOUR;
+	 if (strncasecmp(info.ciphers[i], "NUL", 3) == 0)
+	    cipher_priority[j++] = GNUTLS_CIPHER_NULL;
       }
       cipher_priority[j] = 0;
    }
